using System;
using System.Drawing;
using System.Collections;
using System.ComponentModel;
using System.Windows.Forms;
using System.Data;
using Syncfusion.Windows.Forms.Grid;

namespace SummaryRowSample
{
	public class GridSummaryRow
	{
		public GridSummaryRow()
		{
		}

		//the summary grid
		private GridControl summaryGrid = null;

		//the main grid
		private GridControlBase grid;

		//used to maintain the summary type
		private SummaryRowCellTag[] cellTags;
	
		#region wiring the main grid
		public void WireSummaryRow(GridControlBase grid, object[] summaryTypes)
		{
			if(summaryGrid != null)
				throw new ArgumentException("grid already wired");

			this.grid = grid;

			GridStyleInfo[] styles = new GridStyleInfo[this.grid.Model.ColCount];
			for(int i = 0; i < this.grid.Model.ColCount; ++i)
			{
				styles[i] = new GridStyleInfo((GridStyleInfoStore)this.grid.Model[1, i].Store.Clone());
				styles[i].ResetCellValue();
			}

			this.WireSummaryRow(grid, summaryTypes, styles);
		}

		public void WireSummaryRow(GridControlBase grid, object[] summaryTypes, GridStyleInfo[] styles)
		{
			if(summaryGrid != null)
				throw new ArgumentException("grid already wired");

			this.grid = grid;

			this.summaryGrid = new GridControl();
			this.summaryGrid.ColCount = grid.Model.ColCount;
			this.summaryGrid.RowCount = 1;
			this.summaryGrid.Rows.Hidden[0] = true;
			this.summaryGrid.VScrollBehavior = GridScrollbarMode.Disabled;
			this.summaryGrid.HScrollBehavior = GridScrollbarMode.Disabled;
			this.summaryGrid.Enabled = false;
			this.summaryGrid.TabStop = false;

			this.summaryGrid.DefaultRowHeight = this.grid.DefaultRowHeight;
			this.summaryGrid.TableStyle = this.grid.Model.TableStyle;

			this.summaryGrid.Model.QueryColWidth += new GridRowColSizeEventHandler(summary_QueryColWidth);
			this.summaryGrid.PrepareViewStyleInfo += new GridPrepareViewStyleInfoEventHandler(summary_PrepareViewStyleInfo);

			this.grid.Model.SaveCellInfo += new GridSaveCellInfoEventHandler(grid_SaveCellInfo);
			this.grid.Model.ColWidthsChanged += new GridRowColSizeChangedEventHandler(grid_ColWidthsChanged);
			this.grid.Layout += new LayoutEventHandler(grid_Layout);
			this.grid.LeftColChanged += new GridRowColIndexChangedEventHandler(grid_LeftColChanged);
			this.grid.Model.ColsMoved += new GridRangeMovedEventHandler(grid_ColsMoved);

			this.grid.Controls.Add(this.summaryGrid);

			SiteSummaryRow();

			this.cellTags = new SummaryRowCellTag[this.grid.Model.ColCount];
			for(int i = 0; i < this.grid.Model.ColCount; ++i)
			{
				this.summaryGrid[1, i + 1].Description = i.ToString();
				this.cellTags[i] = new SummaryRowCellTag(summaryTypes[i], styles[i]);
			}

			CalculateValues();

			this.grid.Refresh();
			
		}
		#endregion

		#region helper methods

		public void CalculateValues()
		{
			for(int i = 0; i < this.grid.Model.ColCount; ++i)
			{
				int k = int.Parse((string)this.summaryGrid[1, i + 1].Description);
				SummaryRowCellTag o = this.cellTags[k] as SummaryRowCellTag;
				if(o != null)
				{

					CustomSummaryCalculation calc = o.Tag as CustomSummaryCalculation;
					if(calc != null)
					{
						this.summaryGrid[1, i + 1].CellValue = calc();
					}
					else
					{
						SummaryType type = (SummaryType)o.Tag;
						if(type == SummaryType.None)
						{
							this.summaryGrid[1, i + 1].Text = "";
						}
						else if(type == SummaryType.Sum)
						{
							double d = 0;
							try
							{
								for(int row = 1; row <= this.grid.Model.RowCount; ++row)
								{
									string s = this.grid.Model[row, i + 1].Text;
									if(s.Length == 0)
										s = "0";
									d += double.Parse(s);
								}
							}
							catch
							{
								d = 0;
							}

							this.summaryGrid[1, i + 1].CellValue = d;
							this.summaryGrid[1, i + 1].Format = this.grid.Model[1, i + 1].Format;
							if(this.grid.Model[1, i + 1].CellValueType != null)
								this.summaryGrid[1, i + 1].CellValueType = this.grid.Model[1, i + 1].CellValueType;

						}
						else
						{
							this.summaryGrid[1, i + 1].Text = "unset";

						}
					}
				}
			}
		}

		public void SiteSummaryRow()
		{
			this.grid.ResetGridBounds();

			this.grid.GridBounds = new Rectangle(this.grid.GridBounds.X,
				this.grid.GridBounds.Y,
				this.grid.GridBounds.Width,
				this.grid.GridBounds.Height - this.grid.DefaultRowHeight);
			int height = this.grid.GridBounds.Height + this.grid.DefaultRowHeight;

			this.summaryGrid.Location = new Point(0, height - this.grid.DefaultRowHeight);

			this.summaryGrid.Size = new Size(this.grid.ClientSize.Width, this.grid.DefaultRowHeight);
		}

		#endregion

		#region  Event Handlers on the main grid
		private void grid_SaveCellInfo(object sender, GridSaveCellInfoEventArgs e)
		{
			int k = int.Parse((string)this.summaryGrid[1, e.ColIndex].Description);
			SummaryRowCellTag o = this.cellTags[k] as SummaryRowCellTag;
			if(o != null)
			{
				CustomSummaryCalculation calc = o.Tag as CustomSummaryCalculation;
				if(calc != null)
				{
					this.summaryGrid[1, e.ColIndex].CellValue = calc();
				}
				else
				{
					SummaryType type = (SummaryType)o.Tag;
					if(type == SummaryType.None)
					{
						this.summaryGrid[1, e.ColIndex].Text = "";
					}
					else if(type == SummaryType.Sum)
					{
						double d = 0;
						try
						{
							for(int row = 1; row <= this.grid.Model.RowCount; ++row)
							{
								string s = (row == e.RowIndex)
									? e.Style.Text : this.grid.Model[row, e.ColIndex].Text;
								 d += double.Parse((s.Length > 0) ? s : "0");
							}
						}
						catch
						{
							d = 0;
						}

						this.summaryGrid[1, e.ColIndex].CellValue = d;
					}
					else
					{
						this.summaryGrid[1, e.ColIndex].Text = "unset";

					}
				}
			}
		}

		private void grid_ColsMoved(object sender, GridRangeMovedEventArgs e)
		{
			this.summaryGrid.Cols.MoveRange(e.From, e.Count, e.Target);
		}

		private void grid_ColWidthsChanged(object sender, GridRowColSizeChangedEventArgs e)
		{
			this.summaryGrid.Refresh();
		}

		private void grid_Layout(object sender, LayoutEventArgs e)
		{
			SiteSummaryRow();
		}

		private void grid_LeftColChanged(object sender, GridRowColIndexChangedEventArgs e)
		{
			this.summaryGrid.LeftColIndex = this.grid.LeftColIndex;
		}
		#endregion

		#region Event handlers on the Summary Grid

		private void summary_PrepareViewStyleInfo(object sender, GridPrepareViewStyleInfoEventArgs e)
		{
	
			if(e.ColIndex > 0 && e.RowIndex > 0)
			{
				int k = int.Parse(e.Style.Description);
				SummaryRowCellTag o = this.cellTags[k] as SummaryRowCellTag;
				if(o != null && o.Style != null)
				{
					e.Style.ModifyStyle(o.Style, Syncfusion.Styles.StyleModifyType.Override);
				}
				e.Style.Borders.Top = new GridBorder(GridBorderStyle.Solid);
			}
			else if(e.ColIndex == 0 && e.RowIndex > 0)
			{
				e.Style.Text = " ";
				e.Style.Interior = this.grid.Model[1, 0].Interior;
			}
		}

		private void summary_QueryColWidth(object sender, GridRowColSizeEventArgs e)
		{
			e.Size = this.grid.Model.ColWidths[e.Index];
			e.Handled = true;
		}

		#endregion

		#region support enums, delegates and classes
		public enum SummaryType
		{
			None,
			Sum
		}

		public delegate object CustomSummaryCalculation();

		public class SummaryRowCellTag //: ICloneable
		{
			private object tag1;
			private GridStyleInfo style1;

			public SummaryRowCellTag(object tag, GridStyleInfo style)
			{
				this.tag1 = tag;
				this.style1 = style;
			}
			public object Tag
			{
				get{return tag1;}
				set {tag1 = value;}
			}
			public GridStyleInfo Style
			{
				get{return style1;}
				set {style1 = value;}
			}

		}
		#endregion
	}
}
