
Imports System
Imports System.Drawing
Imports System.Collections
Imports System.ComponentModel
Imports System.Windows.Forms
Imports System.Data
Imports Syncfusion.Windows.Forms.Grid



Public Class GridSummaryRow
   
   Public Sub New()
   End Sub 'New
   
   'the summary grid
   Private summaryGrid As GridControl = Nothing
   
   'the main grid
   Private grid As GridControlBase
   
   'used to maintain the summary type
   Private cellTags() As SummaryRowCellTag
   
   #Region "wiring the main grid"
   
   Overloads Public Sub WireSummaryRow(grid As GridControlBase, summaryTypes() As Object)
      If Not (summaryGrid Is Nothing) Then
         Throw New ArgumentException("grid already wired")
      End If 
      Me.grid = grid
      
      Dim styles(Me.grid.Model.ColCount) As GridStyleInfo
      Dim i As Integer
      
      While i < Me.grid.Model.ColCount
         styles(i) = New GridStyleInfo(CType(Me.grid.Model(1, i).Store.Clone(), GridStyleInfoStore))
         styles(i).ResetCellValue()
         i += 1
      End While
      
      Me.WireSummaryRow(grid, summaryTypes, styles)
   End Sub 'WireSummaryRow
   
   
   Overloads Public Sub WireSummaryRow(grid As GridControlBase, summaryTypes() As Object, styles() As GridStyleInfo)
      If Not (summaryGrid Is Nothing) Then
         Throw New ArgumentException("grid already wired")
      End If 
      Me.grid = grid
      
      Me.summaryGrid = New GridControl()
      Me.summaryGrid.ColCount = grid.Model.ColCount
      Me.summaryGrid.RowCount = 1
      Me.summaryGrid.Rows.Hidden(0) = True
      Me.summaryGrid.VScrollBehavior = GridScrollbarMode.Disabled
      Me.summaryGrid.HScrollBehavior = GridScrollbarMode.Disabled
      Me.summaryGrid.Enabled = False
      Me.summaryGrid.TabStop = False
      
      Me.summaryGrid.DefaultRowHeight = Me.grid.DefaultRowHeight
      Me.summaryGrid.TableStyle = Me.grid.Model.TableStyle
      
      AddHandler Me.summaryGrid.Model.QueryColWidth, AddressOf summary_QueryColWidth
      AddHandler Me.summaryGrid.PrepareViewStyleInfo, AddressOf summary_PrepareViewStyleInfo
      
      AddHandler Me.grid.Model.SaveCellInfo, AddressOf grid_SaveCellInfo
      AddHandler Me.grid.Model.ColWidthsChanged, AddressOf grid_ColWidthsChanged
      AddHandler Me.grid.Layout, AddressOf grid_Layout
      AddHandler Me.grid.LeftColChanged, AddressOf grid_LeftColChanged
      AddHandler Me.grid.Model.ColsMoved, AddressOf grid_ColsMoved
      
      Me.grid.Controls.Add(Me.summaryGrid)
      
      SiteSummaryRow()
      
      Me.cellTags = New SummaryRowCellTag(Me.grid.Model.ColCount) {}
      Dim i As Integer
      
      While i < Me.grid.Model.ColCount
         Me.summaryGrid(1, i + 1).Description = i.ToString()
         Me.cellTags(i) = New SummaryRowCellTag(summaryTypes(i), styles(i))
         i += 1
      End While
      
      CalculateValues()
      
      Me.grid.Refresh()
   End Sub 'WireSummaryRow 
   #End Region
   
   #Region "helper methods"
   
   
   Public Sub CalculateValues()
      Dim i As Integer
      
      While i < Me.grid.Model.ColCount
         Dim k As Integer = Integer.Parse(CStr(Me.summaryGrid(1, i + 1).Description))
         Dim o As SummaryRowCellTag = Me.cellTags(k) '
          If Not (o Is Nothing) Then

                If (o.Tag.GetType() Is GetType(CustomSummaryCalculation)) Then
                    Dim calc As CustomSummaryCalculation = CType(o.Tag, CustomSummaryCalculation) '
                        If Not (calc Is Nothing) Then
                        Me.summaryGrid(1, i + 1).CellValue = calc()
                    End If

                Else
                    Dim type As SummaryType = CType(o.Tag, SummaryType)
                    If type = SummaryType.None Then
                        Me.summaryGrid(1, i + 1).Text = ""
                    ElseIf type = SummaryType.Sum Then
                        Dim d As Double = 0
                        Try
                            Dim row As Integer = 1

                            While row <= Me.grid.Model.RowCount
                                Dim s As String = Me.grid.Model(row, i + 1).Text
                                If s.Length = 0 Then
                                    s = "0"
                                End If
                                d += Double.Parse(s)
                                row += 1
                            End While
                        Catch
                            d = 0
                        End Try

                        Me.summaryGrid(1, i + 1).CellValue = d
                        Me.summaryGrid(1, i + 1).Format = Me.grid.Model(1, i + 1).Format
                        If Not Me.grid.Model(1, i + 1).CellValueType Is Nothing Then
                            Me.summaryGrid(1, i + 1).CellValueType = Me.grid.Model(1, i + 1).CellValueType
                        End If
                    Else
                        Me.summaryGrid(1, i + 1).Text = "unset"
                    End If
                End If
            End If
            i += 1
        End While
   End Sub 'CalculateValues
   
   
   Public Sub SiteSummaryRow()
      Me.grid.ResetGridBounds()
      
      Me.grid.GridBounds = New Rectangle(Me.grid.GridBounds.X, Me.grid.GridBounds.Y, Me.grid.GridBounds.Width, Me.grid.GridBounds.Height - Me.grid.DefaultRowHeight)
      Dim height As Integer = Me.grid.GridBounds.Height + Me.grid.DefaultRowHeight
      
      Me.summaryGrid.Location = New Point(0, height - Me.grid.DefaultRowHeight)
      
      Me.summaryGrid.Size = New Size(Me.grid.ClientSize.Width, Me.grid.DefaultRowHeight)
   End Sub 'SiteSummaryRow
   
   #End Region
   
   #Region " Event Handlers on the main grid"
   
   Private Sub grid_SaveCellInfo(sender As Object, e As GridSaveCellInfoEventArgs)
      Dim k As Integer = Integer.Parse(CStr(Me.summaryGrid(1, e.ColIndex).Description))
      Dim o As SummaryRowCellTag = Me.cellTags(k) '
        If Not (o Is Nothing) Then

            If (o.Tag.GetType() Is GetType(CustomSummaryCalculation)) Then
                Dim calc As CustomSummaryCalculation = CType(o.Tag, CustomSummaryCalculation) '
                If Not (calc Is Nothing) Then
                    Me.summaryGrid(1, e.ColIndex).CellValue = calc()
                End If
            Else
                Dim type As SummaryType = CType(o.Tag, SummaryType)
                If type = SummaryType.None Then
                    Me.summaryGrid(1, e.ColIndex).Text = ""
                ElseIf type = SummaryType.Sum Then
                    Dim d As Double = 0
                    Try
                        Dim row As Integer = 1

                        While row <= Me.grid.Model.RowCount
                            Dim s As String
                            If (row = e.RowIndex) Then
                                s = e.Style.Text
                            Else
                                s = Me.grid.Model(row, e.ColIndex).Text
                            End If
                            d += Double.Parse(IIf(s.Length > 0, s, "0"))
                            row += 1
                        End While
                    Catch
                        d = 0
                    End Try

                    Me.summaryGrid(1, e.ColIndex).CellValue = d
                Else
                    Me.summaryGrid(1, e.ColIndex).Text = "unset"
                End If
            End If
        End If
   End Sub 'grid_SaveCellInfo
   
   
   Private Sub grid_ColsMoved(sender As Object, e As GridRangeMovedEventArgs)
      Me.summaryGrid.Cols.MoveRange(e.From, e.Count, e.Target)
   End Sub 'grid_ColsMoved
   
   
   Private Sub grid_ColWidthsChanged(sender As Object, e As GridRowColSizeChangedEventArgs)
      Me.summaryGrid.Refresh()
   End Sub 'grid_ColWidthsChanged
   
   
   Private Sub grid_Layout(sender As Object, e As LayoutEventArgs)
      SiteSummaryRow()
   End Sub 'grid_Layout
   
   
   Private Sub grid_LeftColChanged(sender As Object, e As GridRowColIndexChangedEventArgs)
      Me.summaryGrid.LeftColIndex = Me.grid.LeftColIndex
   End Sub 'grid_LeftColChanged
   #End Region
   
   #Region "Event handlers on the Summary Grid"
   
   
   Private Sub summary_PrepareViewStyleInfo(sender As Object, e As GridPrepareViewStyleInfoEventArgs)
      
      If e.ColIndex > 0 AndAlso e.RowIndex > 0 Then
         Dim k As Integer = Integer.Parse(e.Style.Description)
         Dim o As SummaryRowCellTag = Me.cellTags(k) '
         'ToDo: Error processing original source shown below
         '    int k = int.Parse(e.Style.Description);
         '    SummaryRowCellTag o = this.cellTags[k] as SummaryRowCellTag;
         '--------------------------------------------^--- Syntax error: ';' expected
         If Not (o Is Nothing) AndAlso Not (o.Style Is Nothing) Then
            e.Style.ModifyStyle(o.Style, Syncfusion.Styles.StyleModifyType.Override)
         End If
         e.Style.Borders.Top = New GridBorder(GridBorderStyle.Solid)
      ElseIf e.ColIndex = 0 AndAlso e.RowIndex > 0 Then
         e.Style.Text = " "
         e.Style.BackColor = Me.grid.Model(1, 0).BackColor
      End If
   End Sub 'summary_PrepareViewStyleInfo
   
   
   Private Sub summary_QueryColWidth(sender As Object, e As GridRowColSizeEventArgs)
      e.Size = Me.grid.Model.ColWidths(e.Index)
      e.Handled = True
   End Sub 'summary_QueryColWidth
   
   #End Region
   
   #Region "support enums, delegates and classes"
   
   Public Enum SummaryType
      None
      Sum
   End Enum 'SummaryType
   
   
   Delegate Function CustomSummaryCalculation() As Object
   
   
   Public Class SummaryRowCellTag ': ICloneable
      Private tag1 As Object
      Private style1 As GridStyleInfo
      
      
      Public Sub New(tag As Object, style As GridStyleInfo)
         Me.tag1 = tag
         Me.style1 = style
      End Sub 'New
      
      Public Property Tag() As Object
         Get
            Return tag1
         End Get
         Set
            tag1 = value
         End Set
      End Property
      
      Public Property Style() As GridStyleInfo
         Get
            Return style1
         End Get
         Set
            style1 = value
         End Set
      End Property 
   End Class 'SummaryRowCellTag
   #End Region
End Class 'GridSummaryRow